/* $Id: testgrid.cpp,v 1.2 2006/01/20 21:42:05 cvsroot Exp $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <iostream>
#include <fstream>
#include "grid.h"

using namespace std;
using namespace GenLib2;


int main(int argc, char* argv[])
{
	fstream in;
	char *fname = NULL;

	struct point p1 = { 0, 0, 0 };
	float ax = 0, ay = 0;

	int i = 0, j = 0;
	while (argc > ++i)
		{
		if (**(argv + i) == '-')
 			{
			if (strcmp(argv[i], "-point") == 0)
				{
				p1.x = atof(argv[++i]);
				p1.y = atof(argv[++i]);
				ax = atof(argv[++i]);
				ay = atof(argv[++i]);
				}
			else
				{
				cerr << "Illegal option: " << argv[i] << endl;
				return 1;
				}
			}
		else
			{
			switch (j++)
				{
				case 0:
					fname = argv[i];
					break;

				default:
					cerr << "Too many arguments given: " << argv[i] << endl;
					exit(1);
				}
			}
		}

	// open input file
	in.open(fname, ios_base::in);
	if (!in)
		{
		cerr << "Cannot open file: " << fname << endl;
		exit(1);
		}


	// find range in x and y
	struct point p;
	in >> p.x >> p.y >> p.c;

	struct range rx, ry;
	rx.min = p.x;
	rx.max = p.x;
	ry.min = p.y;
	ry.max = p.y;

	while (in >> p.x >> p.y >> p.c)
		{
		if (p.x < rx.min) rx.min = p.x;
		if (p.x > rx.max) rx.max = p.x;
		if (p.y < ry.min) ry.min = p.y;
		if (p.y > ry.max) ry.max = p.y;
		}

	cout << "Range\n";
	cout << "  in x: " << rx.min << ", " << rx.max << "\n";
	cout << "  in y: " << ry.min << ", " << ry.max << "\n";

	Grid points;
	points.set(rx, ry);
	//Vector2<float> p0(0, 16);
	//points.set(p0, 2);
	//points.set(0., 0., 2.);
	cout << "Grid parameter:" << endl;
	points.info(cout);

	// put points in the grid
	in.clear();
	in.seekg(ios_base::beg);

	while (in >> p.x >> p.y >> p.c)
		points.insert(p);

	cout << "Number of points: " << points.size() << endl;

	cout << "Data points:\n";
	points.print(cout);
	cout << endl;

	cout << "Occupation:\n";
	points.occupied(cout);
	cout << endl;

	// search
	struct rect range;
	range.x1 = p1.x - ax;
	range.x2 = p1.x + ax;
	range.y1 = p1.y - ay;
	range.y2 = p1.y + ay;

	cout << "Point   : " << p1.x << " " << p1.y << endl;
	cout << "Radius  : " << ax << " " << ay << endl;
	cout << "Range in x : " << range.x1 << ", " << range.x2 << endl;
	cout << "      in y : " << range.y1 << ", " << range.y2 << endl;
	cout << endl;

	//int n = points.search(range);
	//cout << "Number of points inside range: " << n << endl;
	//cout << endl;

	// closest point
	struct point *c = points.closest(range, p1);
	cout << "Closest point: ";
	if (c != 0)
		cout << c->x << " " << c->y << " " << c->c << endl;
	else
		cout << "not found!" << endl;
	cout << endl;

	// interpolate
	int n = points.average(range, p1);
	cout << "Average : ";
	if (n > 0)
		cout << p1.c << " of " << n << " points" << endl;
	else
		 cout << "nothing inside range!" << endl;
	cout << endl;

	cout << "Clear grid ..." << endl;
	points.clear();
	cout << "Number of points: " << points.size() << endl;

	return 0;
}
