/* $Id: osform.h,v 1.3 2006/01/18 12:01:33 cvsroot Exp $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_OSFORM_INCLUDED
#define GENLIB2_OSFORM_INCLUDED

#include <cstdio> // required for MS VC++ 6.0
#include <cstdarg>

namespace GenLib2
{

/** Replacement of the old-style output function form.
 *  cout << form() is equivalent to printf() in C.
 *  This function will be produce ugly output,
 *  obviously when the allocated string buffer
 *  is not large enough.
 *
 *  Use the unsafe function vsprintf when vsnprintf
 *  not exist. vsprintf might be exist on more systems.
 *  The output string is limited to 512 characters
 *  (including the trailing zero).
 *
 *  See:
 *  \begin{itemize}
 *   \item Stroustrup, Bjarne: C++, Addision-Wesley GmbH,
 *     2. unver�derter Nachdruck, 1990, page 278.
 *   \item GNU manual page for vprintf.
 *   \item MS Visual Studio
 *  \end{itemize}
 *
 *  @param fmt format string, format see printf
 *  @return character string
 *  @author Dirk Stallmann
 */
extern char* form(const char* fmt, ...);

} // namespace GenLib2

#endif /* GENLIB2_OSFORM_INCLUDED */

